/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.syndesis.project.converter.visitor;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.integration.Step;
import java.util.Objects;
import java.util.Optional;
import java.util.Queue;
import java.util.Set;
import java.util.function.Function;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link StepVisitorContext}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new StepVisitorContext.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableStepVisitorContext.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "StepVisitorContext"})
final class ImmutableStepVisitorContext
    implements StepVisitorContext {
  private final int index;
  private final Step step;
  private final Queue<Step> remaining;
  private final Function<Step, Optional<String>> connectorIdSupplier;

  private ImmutableStepVisitorContext(
      int index,
      Step step,
      Queue<Step> remaining,
      Function<Step, Optional<String>> connectorIdSupplier) {
    this.index = index;
    this.step = step;
    this.remaining = remaining;
    this.connectorIdSupplier = connectorIdSupplier;
  }

  /**
   * @return The value of the {@code index} attribute
   */
  @JsonProperty("index")
  @Override
  public int getIndex() {
    return index;
  }

  /**
   * @return The value of the {@code step} attribute
   */
  @JsonProperty("step")
  @Override
  public Step getStep() {
    return step;
  }

  /**
   * @return The value of the {@code remaining} attribute
   */
  @JsonProperty("remaining")
  @Override
  public Queue<Step> getRemaining() {
    return remaining;
  }

  /**
   * @return The value of the {@code connectorIdSupplier} attribute
   */
  @JsonProperty("connectorIdSupplier")
  @Override
  public Function<Step, Optional<String>> getConnectorIdSupplier() {
    return connectorIdSupplier;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link StepVisitorContext#getIndex() index} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for index
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableStepVisitorContext withIndex(int value) {
    if (this.index == value) return this;
    return validate(new ImmutableStepVisitorContext(value, this.step, this.remaining, this.connectorIdSupplier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link StepVisitorContext#getStep() step} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for step (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableStepVisitorContext withStep(Step value) {
    if (this.step == value) return this;
    return validate(new ImmutableStepVisitorContext(this.index, value, this.remaining, this.connectorIdSupplier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link StepVisitorContext#getRemaining() remaining} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for remaining (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableStepVisitorContext withRemaining(Queue<Step> value) {
    if (this.remaining == value) return this;
    return validate(new ImmutableStepVisitorContext(this.index, this.step, value, this.connectorIdSupplier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link StepVisitorContext#getConnectorIdSupplier() connectorIdSupplier} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for connectorIdSupplier (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableStepVisitorContext withConnectorIdSupplier(Function<Step, Optional<String>> value) {
    if (this.connectorIdSupplier == value) return this;
    return validate(new ImmutableStepVisitorContext(this.index, this.step, this.remaining, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableStepVisitorContext} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableStepVisitorContext
        && equalTo((ImmutableStepVisitorContext) another);
  }

  private boolean equalTo(ImmutableStepVisitorContext another) {
    return index == another.index
        && Objects.equals(step, another.step)
        && Objects.equals(remaining, another.remaining)
        && Objects.equals(connectorIdSupplier, another.connectorIdSupplier);
  }

  /**
   * Computes a hash code from attributes: {@code index}, {@code step}, {@code remaining}, {@code connectorIdSupplier}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + index;
    h += (h << 5) + Objects.hashCode(step);
    h += (h << 5) + Objects.hashCode(remaining);
    h += (h << 5) + Objects.hashCode(connectorIdSupplier);
    return h;
  }

  /**
   * Prints the immutable value {@code StepVisitorContext} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "StepVisitorContext{"
        + "index=" + index
        + ", step=" + step
        + ", remaining=" + remaining
        + ", connectorIdSupplier=" + connectorIdSupplier
        + "}";
  }

  /**
   * Construct a new immutable {@code StepVisitorContext} instance.
   * @param index The value for the {@code index} attribute
   * @param step The value for the {@code step} attribute
   * @param remaining The value for the {@code remaining} attribute
   * @param connectorIdSupplier The value for the {@code connectorIdSupplier} attribute
   * @return An immutable StepVisitorContext instance
   */
  public static ImmutableStepVisitorContext of(int index, Step step, Queue<Step> remaining, Function<Step, Optional<String>> connectorIdSupplier) {
    return validate(new ImmutableStepVisitorContext(index, step, remaining, connectorIdSupplier));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableStepVisitorContext validate(ImmutableStepVisitorContext instance) {
    Set<ConstraintViolation<ImmutableStepVisitorContext>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link StepVisitorContext} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable StepVisitorContext instance
   */
  public static ImmutableStepVisitorContext copyOf(StepVisitorContext instance) {
    if (instance instanceof ImmutableStepVisitorContext) {
      return (ImmutableStepVisitorContext) instance;
    }
    return new StepVisitorContext.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableStepVisitorContext ImmutableStepVisitorContext}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private int index;
    private Step step;
    private Queue<Step> remaining;
    private Function<Step, Optional<String>> connectorIdSupplier;

    /**
     * Creates a builder for {@link ImmutableStepVisitorContext ImmutableStepVisitorContext} instances.
     */
    Builder() {
      if (!(this instanceof StepVisitorContext.Builder)) {
        throw new UnsupportedOperationException("Use: new StepVisitorContext.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code StepVisitorContext} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final StepVisitorContext.Builder createFrom(StepVisitorContext instance) {
      Objects.requireNonNull(instance, "instance");
      index(instance.getIndex());
      Step stepValue = instance.getStep();
      if (stepValue != null) {
        step(stepValue);
      }
      Queue<Step> remainingValue = instance.getRemaining();
      if (remainingValue != null) {
        remaining(remainingValue);
      }
      Function<Step, Optional<String>> connectorIdSupplierValue = instance.getConnectorIdSupplier();
      if (connectorIdSupplierValue != null) {
        connectorIdSupplier(connectorIdSupplierValue);
      }
      return (StepVisitorContext.Builder) this;
    }

    /**
     * Initializes the value for the {@link StepVisitorContext#getIndex() index} attribute.
     * @param index The value for index 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("index")
    public final StepVisitorContext.Builder index(int index) {
      this.index = index;
      return (StepVisitorContext.Builder) this;
    }

    /**
     * Initializes the value for the {@link StepVisitorContext#getStep() step} attribute.
     * @param step The value for step (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("step")
    public final StepVisitorContext.Builder step(Step step) {
      this.step = step;
      return (StepVisitorContext.Builder) this;
    }

    /**
     * Initializes the value for the {@link StepVisitorContext#getRemaining() remaining} attribute.
     * @param remaining The value for remaining (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("remaining")
    public final StepVisitorContext.Builder remaining(Queue<Step> remaining) {
      this.remaining = remaining;
      return (StepVisitorContext.Builder) this;
    }

    /**
     * Initializes the value for the {@link StepVisitorContext#getConnectorIdSupplier() connectorIdSupplier} attribute.
     * @param connectorIdSupplier The value for connectorIdSupplier (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connectorIdSupplier")
    public final StepVisitorContext.Builder connectorIdSupplier(Function<Step, Optional<String>> connectorIdSupplier) {
      this.connectorIdSupplier = connectorIdSupplier;
      return (StepVisitorContext.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableStepVisitorContext ImmutableStepVisitorContext}.
     * @return An immutable instance of StepVisitorContext
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableStepVisitorContext build() {
      return ImmutableStepVisitorContext.validate(new ImmutableStepVisitorContext(index, step, remaining, connectorIdSupplier));
    }
  }
}
